"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createServer = createServer;

var _koa = _interopRequireDefault(require("koa"));

var _path = _interopRequireDefault(require("path"));

var _http = _interopRequireDefault(require("http"));

var _http2 = _interopRequireDefault(require("http2"));

var _fs = _interopRequireDefault(require("fs"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _net = _interopRequireDefault(require("net"));

var _createMiddlewares = require("./create-middlewares.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A request handler that returns a 301 HTTP Redirect to the same location as the original
 * request but using the https protocol
 */
function httpsRedirect(req, res) {
  const {
    host
  } = req.headers;
  res.writeHead(301, {
    Location: `https://${host}${req.url}`
  });
  res.end();
}
/**
 * Creates a koa server with middlewares, but does not start it. Returns the koa app and
 * http server instances.
 *
 * @param {import('./config').InternalConfig} cfg the server configuration
 * @param {import('chokidar').FSWatcher} fileWatcher
 * @returns {{ app: import('koa'), server: net.Server | httpServer.Server }}
 */


function createServer(cfg, fileWatcher = _chokidar.default.watch([])) {
  const middlewares = (0, _createMiddlewares.createMiddlewares)(cfg, fileWatcher);
  const app = new _koa.default();
  middlewares.forEach(middleware => {
    app.use(middleware);
  });
  let server;

  if (cfg.http2) {
    const dir = _path.default.join(__dirname, '..');

    const options = {
      key: _fs.default.readFileSync(cfg.sslKey ? cfg.sslKey : _path.default.join(dir, '.self-signed-dev-server-ssl.key')),
      cert: _fs.default.readFileSync(cfg.sslCert ? cfg.sslCert : _path.default.join(dir, '.self-signed-dev-server-ssl.cert')),
      allowHTTP1: true
    };

    const httpsRedirectServer = _http.default.createServer(httpsRedirect);

    const appServer = _http2.default.createSecureServer(options, app.callback());

    let appServerPort;
    let httpsRedirectServerPort;
    /**
     * A connection handler that checks if the connection is using TLS
     */

    const httpRedirectProxy = connection => {
      connection.once('data', buffer => {
        // A TLS handshake record starts with byte 22.
        const address = buffer[0] === 22 ? appServerPort : httpsRedirectServerPort;

        const proxy = _net.default.createConnection(address, () => {
          proxy.write(buffer);
          connection.pipe(proxy).pipe(connection);
        });
      });
    };

    server = _net.default.createServer(httpRedirectProxy);
    server.addListener('close', () => {
      httpsRedirectServer.close();
      appServer.close();
    });
    server.addListener('listening', () => {
      const {
        address,
        port
      } = server.address();
      appServerPort = port + 1;
      httpsRedirectServerPort = port + 2;
      appServer.listen({
        address,
        port: appServerPort
      });
      httpsRedirectServer.listen({
        address,
        port: httpsRedirectServerPort
      });
    });
    const serverListen = server.listen.bind(server);

    server.listen = (config, callback) => {
      appServer.addListener('listening', callback);
      serverListen(config);
      return server;
    };
  } else {
    server = _http.default.createServer(app.callback());
  }

  return {
    server,
    app
  };
}